pub struct Update<T, M>where
T: Uniplate,{ /* private fields */ }Expand description
Represents the effects of a successful rule application.
Contains the new subtree and any side-effects. This type is not intended to be constructed directly, but rather created by the engine to pass to the user-defined selector functions.
Implementations