pub enum CacheResult<T> {
Unknown,
Terminal(usize),
Rewrite(T),
}Expand description
Return type for RewriteCache Due to the nature of Rewriting, there may be repeated subtrees where no rule can be applied. In that case, we can compute it once and store it in cache stating no rules applicable.
Variants
Unknown
The Subtree does not exist in cache.
Terminal(usize)
The Subtree exists in cache but no rule is applicable.
Rewrite(T)
The Subtree exists in cache and there is a pre computed value.