pub struct NamedRule<F: Clone> { /* private fields */ }Expand description
We can create a rule using this struct and pass it into our list of rules directly,
For debugging and tracing, it is helpful to see rules by a meaningful name.
or we can make use of the named_rule macro (see [tree-morph-macros]).
This struct and macro is for the short form way of defining named rules. You can change the name
of the rule by implementing the Rule trait as well.
use tree_morph::prelude::*;
use tree_morph_macros::named_rule;
use uniplate::Uniplate;
#[derive(Debug, Clone, PartialEq, Eq, Uniplate)]
#[uniplate()]
enum Expr {
// Snip
}
struct Meta;
#[named_rule("CustomName")]
fn my_rule(_: &mut Commands<Expr, Meta>, expr: &Expr, _: &Meta) -> Option<Expr> {
/// rule implementation
}This macro will return a helper function called my_rule which returns the NamedRule for us to
use. We can add this to our list of rules with vec![my_rule].
If a name is not specified, the functions name will be it’s identifier.
Implementations